library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity faser is
  Port ( takt_I       : in std_logic;
         reset_I      : in std_logic;
	    temperatur_I : in std_logic;
	    emergency_I  : in std_logic;
	    laseron_I    : in std_logic;
	    amplifier_I  : in std_logic;
	    status_I     : in std_logic;
	    pwrld_I      : in std_logic;
	    daten_I      : in std_logic_vector(7 downto 0);
         pwrld_O      : out std_logic;
	    laseron_O    : out std_logic;
	    status_O     : out std_logic;
	    reset_O      : out std_logic;
	    ro_O         : out std_logic;
	    frequenz_O   : out std_logic);
end faser;

architecture Behavioral of faser is
-- interner Zwischenspeicher
signal q      : std_logic_vector(31 downto 0);
signal enable : std_logic;
signal reset  : std_logic;
signal ro     : std_logic;
signal reload : std_logic;
signal freqin : std_logic_vector(7 downto 0);
signal freq   : std_logic_vector(7 downto 0);
signal fout   : std_logic;
signal xg     : std_logic;
signal xk     : std_logic;
 
-- Komponenten einbinden
component ct8
    Port ( takt : in std_logic;
           load : in std_logic;
           d : in std_logic_vector(7 downto 0);
           q : out std_logic_vector(7 downto 0));
end component ct8;

component ct32
    Port ( takt : in std_logic;
           reset : in std_logic;
           q : out std_logic_vector(31 downto 0));
end component ct32;

component rsff
    Port ( s : in std_logic;
           r : in std_logic;  
           q : out std_logic);       
end component rsff;

component tff
    Port ( clk	: in std_logic;
           rst : in std_logic;  
           q   : out std_logic);       
end component tff;

-- Start
begin
  --instanzieren von Komponenten
  counter_ct32: ct32 PORT MAP(takt_I,reset,q);
  counter_freq: ct8  PORT MAP(takt_I,reload,freqin,freq);
  rsff_ff1:     rsff PORT MAP(enable,reset,ro);
  tff_ff2:      tff  PORT MAP(reload,reset,fout);

  --statische Zuordnung von Logik
  reset <= NOT (reset_I) OR NOT (emergency_I) OR NOT (temperatur_I) OR (amplifier_I AND laseron_I);  
  reset_O <= reset;
  ro_O <= ro;
 
  pwrld_O <= ro AND pwrld_I AND NOT (laseron_I);
  laseron_O <= ro AND status_I AND laseron_I;
  status_O <= ro AND status_I; 

  reload <= NOT(freq(0)) AND NOT(freq(1)) AND NOT(freq(2)) AND NOT(freq(3)) AND 
            NOT(freq(4)) AND NOT(freq(5)) AND NOT(freq(6)) AND NOT(freq(7));
  frequenz_O <= fout;
  xk <= daten_I(5) AND daten_I(6) AND daten_I(7);
  xg <= xk OR 	(NOT(xk) AND daten_I(3) AND daten_I(4));


-- Q0 = (XD0  & XG) # XD7  
-- Q1 = (XD1  & XG) # XD7  
-- Q2 = (XD2  & XG) # XD7
-- Q3 = (XD3  )     # !XG          # XD7 
-- Q4 = (XD4  )     # !XD5 & !XD6  # XD7
-- Q5 = (XD5  )     # XD7  
-- Q6 = (XD6  )     # XD7
-- Q7 = (GND  ) 

  freqin(0) <= (daten_I(0) AND xg)	OR daten_I(7);
  freqin(1) <= (daten_I(1) AND xg) OR daten_I(7);
  freqin(2) <= (daten_I(2) AND xg) OR daten_I(7);
  freqin(3) <=  daten_I(3) OR NOT(xg) or daten_I(7);
  freqin(4) <=  daten_I(4) OR (NOT(daten_I(5)) AND NOT(daten_I(6))) OR daten_I(7);
  freqin(5) <=  daten_I(5) OR daten_I(7);
  freqin(6) <=  daten_I(6) OR daten_I(7);
  freqin(7) <=  '0';

  -- Abfrageprozess, ob Zeit von 3min abgelaufen ist
  count: PROCESS (q)
  BEGIN   
    IF q = "01000000000000000000000000000000" THEN enable <= '1';
                                              ELSE enable <= '0';
    END IF;

  END PROCESS count;
 
end Behavioral;
